import { InvocationType, LambdaConfiguration, LambdaType } from "./type";

const __METADATA_SYMBOL = Symbol.for("__LAMBDA_METADATA__");

export function isLambdaMetadata(val: any): val is LambdaMetadata {
  return val?.__is_metadata === __METADATA_SYMBOL;
}

export class LambdaMetadata implements LambdaConfiguration {
  name: string;
  exportedName: string;
  invocationMethod: InvocationType = "sync";
  role?: string;
  description?: string;
  timeout?: number;
  memory?: number;
  environmentVariables?: Record<string, string>;

  __is_metadata = __METADATA_SYMBOL;

  constructor(configuration: LambdaConfiguration, public path: string = "") {
    Object.assign(this, configuration);
  }
}

export const lambda: LambdaType<LambdaMetadata> = (configuration, fn) => {
  return new LambdaMetadata({
    ...configuration,
    name: configuration.name ?? fn.name,
  });
};
